C------------------------------------------------------------------
      PROGRAM ELQUABE
C
C  PROGRAM 36
C 
C
C  THIS PROGRAM SOLVES TWO DIMENSIONAL (EL)ASTIC PROBLEMS
C  USING (QUA)DRATIC (B)OUNDARY (E)LEMENTS
C
      CHARACTER*10 FILEIN,FILEOUT
C
      COMMON/MATG/ G(100,150)
      COMMON/MATH/ H(100,100)
      COMMON N,L,GE,XNU,INP,IPR
      DIMENSION X(51),Y(51)
      DIMENSION DFI(150),FI(100),KODE(150)
      DIMENSION CX(20),CY(20),SSOL(60),DSOL(40)
C
C  SET MAXIMUN DIMENSION OF THE SYSTEM OF EQUATIONS (NX)
C  NX=  2*MAXIMUN NUMBER OF NODES= 4*MAXIMUN NUMBER OF ELEMENTS
C  NX1= 3*MAXIMUN NUMBER OF NODES= 6*MAXIMUN NUMBER OF ELEMENTS
C
      NX=100
      NX1=150
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN  FILES FOR INPUT AND OUTPUT    
C
      WRITE(*,' (A) ') ' NAME OF INPUT FILE (MAX. 10 CHART.)'
      READ(*,'  (A) ')FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,' (A) ') ' NAME OF OUTPUT FILE (MAX. 10 CHART.)'
      READ(*,' (A) ')FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C 
C  READ DATA
C
      CALL INPUTEQ(CX,CY,X,Y,KODE,DFI)
C
C  COMPUTE G AND H MATRICES AND FORM SYSTEM (A X = F)
C
      CALL GHMATEQ(X,Y,G,H,FI,DFI,KODE,NX,NX1) 
C
C  SOLVE SYSTEM OF EQUATIONS
C
      NN=2*N
      CALL SLNPD(H,FI,D,NN,NX)
C
C  COMPUTE STRESS AND DISPLACEMENT VALUES AT INTERNAL POINTS.
C
      CALL INTEREQ(FI,DFI,KODE,CX,CY,X,Y,SSOL,DSOL)
C
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS
C
      CALL OUTPTEQ(X,Y,FI,DFI,CX,CY,SSOL,DSOL)
C
C   CLOSE INPUT AND OUTPUT FILES
C
      CLOSE (INP)
      CLOSE (IPR)
      STOP
      END
C-----------------------------------------------------------------------
      SUBROUTINE INPUTEQ(CX,CY,X,Y,KODE,DFI)
C
C  PROGRAM 37
C
C  NE= NUMBER OF BOUNDARY ELEMENTS
C  L = NUMBER OF INTERNAL POINTS
C  GE= SHEAR MODULUS
C  XNU= POISSON RATIO
C 
      CHARACTER*80 TITLE
      COMMON N,L,GE,XNU,INP,IPR
      DIMENSION KODE(150),X(51),Y(51),CX(20),CY(20),DFI(150)
      WRITE(IPR,100)
  100 FORMAT(' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(A)') TITLE
C
C  READ NUMBER OF BOUNDARY ELEMENTS AND INTERNAL POINTS;
C  READ MATERIAL PROPERTIES
C
      READ(INP,*)NE,L,GE,XNU
      WRITE(IPR,210)NE,L,GE,XNU
  210 FORMAT(//2X,'DATA'/2X,'NUMBER OF BOUNDARY ELEMENTS=',
     1I3/2X,'NUMBER OF INTERNAL POINTS=',I3/2X,'SHEAR MODULUS=',
     2E14.7/2X,'POISSON RATIO=',E14.7)
      N=2*NE
C
C  READ BOUNDARY NODES COORDINATES    
C
      N=2*NE
      WRITE(IPR,500)
      READ(INP,*) (X(I),Y(I),I=1,N)
      DO 10 I=1,N
   10 WRITE(IPR,240) I,X(I),Y(I)
  500 FORMAT(//2X,'BOUNDARY NODES COORDINATES'///4X,
     1'NODE',10X,'X',18X,'Y'/)
  240 FORMAT(5X,I3,2(5X,E14.7))
C
C  READ BOUNDARY CONDITIONS IN DFI(I) VECTOR, IF KODE(I)=0
C  THE DFI(I) VALUE IS A KNOWN DISPLACEMENT; IF KODE(I)=1 THE
C  DFI(I) VALUE IS A KNOWN TRACTION.
C  SIX BOUNDARY CONDITIONS ARE READ PER ELEMENT.
C  NODES BETWEEN TWO ELEMENTS MAY HAVE TWO DIFFERENT VALUES
C  OF THE TRACTION BUT ONLY ONE VALUE OF THE DISPLACEMENT.
C
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS'//35X,'PRESCRIBED VALUES' /
     17X,'-----FIRST NODE------',3X,'-----SECOND NODE-----',3X,
     2'-----THIRD NODE------'/
     31X,'ELE',3X,'X DIR.',2X,'C',3X,'Y DIR.',2X,'C',3X,
     4'X DIR.',2X,'C',3X,'Y DIR.',2X,'C',3X,'X DIR.',
     52X,'C',3X,'Y DIR.',2X,'C'/)
      DO 20 I=1,NE
      READ(INP,*) (KODE(6*I-6+J),DFI(6*I-6+J),J=1,6)
   20 WRITE(IPR,950)I,(DFI(6*I-6+J),KODE(6*I-6+J),J=1,6)
  950 FORMAT(1X,I3,6(F9.3,2X,I1))               
C
C  READ INTERNAL POINTS COORDINATES
C
      IF(L.EQ.0) GO TO 30
      READ(INP,*) (CX(I),CY(I),I=1,L)
   30 RETURN
      END
C------------------------------------------------------------------------
      SUBROUTINE GHMATEQ(X,Y,G,H,FI,DFI,KODE,NX,NX1)
C
C  PROGRAM 38
C
C  THIS SUBROUTINE COMPUTES THE G AND H MATRICES AND FORMS
C  THE SYSTEM A X = F READY TO BE SOLVED
C  H IS A SQUARE MATRIX (4*NE,4*NE); G IS RECTANGULAR (4*NE,6*NE)
C
      DIMENSION X(51),Y(51),G(NX,NX1),H(NX,NX),HW(2,6),GW(2,6)
      DIMENSION FI(100),DFI(150),KODE(150)
      COMMON N,L,GE,XNU,INP,IPR   
      NN=2*N
      NE=N/2
      DO 20 I=1,NN
      DO 11 J=1,NN
   11 H(I,J)=0.
      DO 12 J=1,3*N
   12 G(I,J)=0.
   20 CONTINUE
      X(N+1)=X(1)
      Y(N+1)=Y(1)
C
C  COMPUTE THE GW AND HW MATRICES FOR EACH COLLOCATION 
C  POINT AND EACH BOUNDARY ELEMENT
C
      DO 40 LL=1,N
      DO 40 I=1,N-1,2
      IF((LL-I)*(LL-I-1)*(LL-I-2)*(LL-I+N-2)) 22,21,22
   21 NODO=LL-I+1
      IF((LL.EQ.1).AND.(I.EQ.N-1)) NODO=NODO+N
      CALL EXTINEQ(X(LL),Y(LL),X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),
     *HW,GW)
      CALL LOCINEQ(X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),GW,NODO)
      GO TO 34
   22 CALL EXTINEQ(X(LL),Y(LL),X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),
     *HW,GW)
C
C  PLUG THE GW AND HW MATRICES INTO THE GENERAL G AND H MATRICES.
C
   34 DO 39 K=1,2
      DO 38 J=1,6
      G(2*LL-2+K,3*I-3+J)=G(2*LL-2+K,3*I-3+J)+GW(K,J)
      IF(I-N+1) 37,35,37
   35 IF(J-5) 37,36,36
   36 H(2*LL-2+K,J-4)=H(2*LL-2+K,J-4)+HW(K,J)      
      GO TO 38
   37 H(2*LL-2+K,2*I-2+J)=H(2*LL-2+K,2*I-2+J)+HW(K,J)
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
C
C  COMPUTE THE DIAGONAL COEFFICIENTS OF THE H MATRIX 
C
      DO 70 I=1,N
      H(2*I-1,2*I-1)=0.
      H(2*I,2*I-1)=0.
      H(2*I-1,2*I)=0.
      H(2*I,2*I)=0.
      DO 60 J=1,N
      IF(I.EQ.J) GO TO 60
      H(2*I-1,2*I-1)=H(2*I-1,2*I-1)-H(2*I-1,2*J-1)
      H(2*I,2*I-1)=H(2*I,2*I-1)-H(2*I,2*J-1)
      H(2*I-1,2*I)=H(2*I-1,2*I)-H(2*I-1,2*J)
      H(2*I,2*I)=H(2*I,2*I)-H(2*I,2*J)
   60 CONTINUE
C
C  ADD ONE TO THE DIAGONAL COEFFICIENTS FOR
C  EXTERNAL PROBLEMS.
C
      IF(H(2*I-1,2*I-1)) 65,70,70
   65 H(2*I-1,2*I-1)=1.+H(2*I-1,2*I-1)
      H(2*I,2*I)=1.+H(2*I,2*I)
   70 CONTINUE
C
C  REORDER THE COLUMNS OF THE SYSTEM OF EQUATIONS IN ACCORDANCE
C  WITH THE BOUNDARY CONDITIONS AND FORM SYSTEM MATRIX A WHICH
C  IS STORED IN H
C
      DO 180 I=1,NE
      DO 170 J=1,6
      IF(KODE(6*I-6+J)) 110,110,170
  110 IF((I-NE).NE.0 .OR. J.LT.5) GO TO 125
      IF(KODE(J-4)) 115,115,113
  113 DO 114 K=1,NN
      CH=H(K,J-4)
      H(K,J-4)=-G(K,6*I-6+J)*GE
  114 G(K,6*I-6+J)=-CH
      GO TO 170
  115 DO 116 K=1,NN
      H(K,J-4)=H(K,J-4)-G(K,6*I-6+J)*GE
  116 G(K,6*I-6+J)=0.
      GO TO 170
  125 IF(I.EQ.1 .OR. J.GT.2 .OR. KODE(6*I-8+J).EQ.1) GO TO 130
      DO 129 K=1,NN
      H(K,4*I-4+J)=H(K,4*I-4+J)-G(K,6*I-6+J)*GE
  129 G(K,6*I-6+J)=0.
      GO TO 170
  130 DO 132 K=1,NN
      CH=H(K,4*I-4+J)
      H(K,4*I-4+J)=-G(K,6*I-6+J)*GE
  132 G(K,6*I-6+J)=-CH
  170 CONTINUE
  180 CONTINUE
C
C  FORM THE RIGHT HAND SIDE VECTOR F WHICH IS STORED IN FI
C
      DO 190 I=1,NN
      FI(I)= 0.
      DO 185 J=1,6*NE
  185 FI(I)=FI(I)+G(I,J)*DFI(J)
  190 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTINEQ(XP,YP,X1,Y1,X2,Y2,X3,Y3,HW,GW)
C
C  PROGRAM 39 
C
C  THIS SUBRUOTINE COMPUTES THE HW AND GW MATRICES 
C  THAT RELATE A NODE (XP,YP) WITH A BOUNDARY
C  ELEMENT USING GAUSS QUADRATURE.
C
C  RA           = RADIUS 
C  RD1,RD2,RDN  = RADIUS DERIVATIVES
C  ETA1,ETA2    = COMPONENTS OF THE UNIT NORMAL TO THE ELEMENT
C  XCO,YCO      = INTEGRATION POINT ALONG THE ELEMENT
C  XJA          = JACOBIAN
C
      COMMON N,L,GE,XNU,INP,IPR 
      DIMENSION GW(2,6),HW(2,6)
      DIMENSION GI(10),OME(10)
      DATA GI/0.9739065285,-0.9739065285,0.8650633666,-0.8650633666
     @,0.6794095683,-0.6794095682,0.4333953941,-0.4333953941,
     @0.1488743389,-0.1488743389/                                    
      DATA OME/0.0666713443,0.0666713443,0.1494513491,0.1494513491
     @,0.2190863625,0.2190863625,0.2692667193,0.2692667193,
     @0.2955242247,0.2955242247/
      DO 30 I=1,2
      DO 20 J=1,6
      HW(I,J)=0.
   20 GW(I,J)=0.
   30 CONTINUE
      A=X3-2*X2+X1
      B=(X3-X1)/2
      C=Y3-2*Y2+Y1
      D=(Y3-Y1)/2 
      DE=4*3.141592*(1-XNU) 
      DO 40 I=1,10
C 
C  COMPUTE THE VALUES OF THE SHAPE FUNCTIONS AT THE
C  INTEGRATION POINTS
C
      F1=GI(I)*(GI(I)-1)*0.5
      F2=1.-GI(I)**2
      F3=GI(I)*(GI(I)+1)*0.5
C
C  COMPUTE GEOMETRICAL PROPERTIES AT THE INTEGRATION POINTS
C 
      XCO=X1*F1+X2*F2+X3*F3
      YCO=Y1*F1+Y2*F2+Y3*F3
      XJA=SQRT((GI(I)*A+B)**2+(GI(I)*C+D)**2)
      ETA1=(GI(I)*C+D)/XJA
      ETA2=-(GI(I)*A+B)/XJA
      RA=SQRT((XP-XCO)**2+(YP-YCO)**2)
      RD1=(XCO-XP)/RA
      RD2=(YCO-YP)/RA
      RDN=RD1*ETA1+RD2*ETA2
C
C  COMPUTE GW AND HW MATRICES
C
      GW(1,1)=GW(1,1)+((3-4*XNU)*ALOG(1./RA)+RD1**2)*OME(I)*XJA*F1/
     1(2*DE*GE)
      GW(1,2)=GW(1,2)+RD1*RD2*OME(I)*XJA*F1/(2*DE*GE)
      GW(2,1)=GW(1,2)                      
      GW(2,2)=GW(2,2)+((3-4*XNU)*ALOG(1./RA)+RD2**2)*OME(I)*XJA*F1/
     1(2*DE*GE)
      HW(1,1)=HW(1,1)-RDN*((1-2*XNU)+2*RD1**2)/(RA*DE)*OME(I)*XJA*F1
      HW(1,2)=HW(1,2)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA1*RD2-ETA2*RD1))*
     1OME(I)*XJA*F1/(RA*DE)
      HW(2,1)=HW(2,1)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA2*RD1-ETA1*RD2))*
     1OME(I)*XJA*F1/(RA*DE)
      HW(2,2)=HW(2,2)-RDN*((1-2*XNU)+2*RD2**2)*OME(I)*XJA*F1/(RA*DE)
      GW(1,3)=GW(1,3)+((3-4*XNU)*ALOG(1./RA)+RD1**2)*OME(I)*XJA*F2/
     1(2*DE*GE)
      GW(1,4)=GW(1,4)+RD1*RD2*OME(I)*XJA*F2/(2*DE*GE)
      GW(2,3)=GW(1,4)                      
      GW(2,4)=GW(2,4)+((3-4*XNU)*ALOG(1./RA)+RD2**2)*OME(I)*XJA*F2/
     1(2*DE*GE)
      HW(1,3)=HW(1,3)-RDN*((1-2*XNU)+2*RD1**2)/(RA*DE)*OME(I)*XJA*F2
      HW(1,4)=HW(1,4)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA1*RD2-ETA2*RD1))*
     1OME(I)*XJA*F2/(RA*DE)
      HW(2,3)=HW(2,3)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA2*RD1-ETA1*RD2))*
     1OME(I)*XJA*F2/(RA*DE)
      HW(2,4)=HW(2,4)-RDN*((1-2*XNU)+2*RD2**2)*OME(I)*XJA*F2/(RA*DE)
      GW(1,5)=GW(1,5)+((3-4*XNU)*ALOG(1./RA)+RD1**2)*OME(I)*XJA*F3/
     1(2*DE*GE)
      GW(1,6)=GW(1,6)+RD1*RD2*OME(I)*XJA*F3/(2*DE*GE)
      GW(2,5)=GW(1,6)                      
      GW(2,6)=GW(2,6)+((3-4*XNU)*ALOG(1./RA)+RD2**2)*OME(I)*XJA*F3/
     1(2*DE*GE)
      HW(1,5)=HW(1,5)-RDN*((1-2*XNU)+2*RD1**2)/(RA*DE)*OME(I)*XJA*F3
      HW(1,6)=HW(1,6)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA1*RD2-ETA2*RD1))*
     1OME(I)*XJA*F3/(RA*DE)
      HW(2,5)=HW(2,5)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA2*RD1-ETA1*RD2))*
     1OME(I)*XJA*F3/(RA*DE)
   40 HW(2,6)=HW(2,6)-RDN*((1-2*XNU)+2*RD2**2)*OME(I)*XJA*F3/(RA*DE)
      RETURN                                                            
      END 
C-----------------------------------------------------------------------
      SUBROUTINE LOCINEQ(XG1,YG1,XG2,YG2,XG3,YG3,GW,NODO)
C
C  PROGRAM 40
C
C  THIS SUBROUTINE COMPUTES THE GW MATRIX WHEN THE COLLOCATION
C  POINT IS ONE OF THE NODES OF THE INTEGRATION ELEMENT.
C  THE COEFFICIENTS ARE COMPUTED BY NUMERICAL INTEGRATION:
C  THE NON SINGULAR PART IS COMPUTED USING STANDARD GAUSS QUADRATURE,
C  THE LOGARITHMIC PART IS COMPUTED USING A SPECIAL QUADRATURE FORMULA.
C
      COMMON N,L,GE,XNU,INP,IPR
      DIMENSION GI(10),OME(10),GIL(10),OMEL(10),GW(2,6),R(2)
C
C  DATA FOR THE GAUSS QUADRATURE
C
      DATA GI/0.9739065285,-0.9739065285,0.8650633666,-0.8650633666      
     @,0.6794095682,-0.6794095682,0.4333953941,-0.4333953941,
     @0.1488743389,-0.1488743389/ 
      DATA OME/0.0666713443,0.0666713443,0.1494513491,0.1494513491 
     @,0.2190863625,0.2190863625,0.2692667193,0.2692667193,
     @0.2955242247,0.2955242247/
C
C  DATA FOR THE SPECIAL QUADRATURE
C
      DATA GIL/0.0090426309,0.0539712662,0.1353118246,0.2470524162
     @,0.3802125396,0.5237923179,0.6657752055,0.7941904160,    
     @0.8981610912,0.9688479887/
      DATA OMEL/0.1209551319,0.1863635425,0.1956608732,0.1735771421
     @,0.1356956729,0.0936467585,0.0557877273,0.0271598109,
     @0.0095151826,0.0016381576/
C
C  SET LOCAL COORDINATES SYSTEM
C
      GOTO(1,2,3),NODO
    1 X3=XG3-XG1
      Y3=YG3-YG1
      X2=XG2-XG1
      Y2=YG2-YG1
      A1=(X3-2*X2)*0.5
      B1=X2 
      A2=(Y3-2*Y2)*0.5
      B2=Y2 
      GO TO 4 
    2 X3=XG3-XG2
      Y3=YG3-YG2
      X1=XG1-XG2
      Y1=YG1-YG2
      A1=X1+X3
      B1=X3-X1
      A2=Y1+Y3
      B2=Y3-Y1
      GO TO 4 
    3 X2=XG2-XG3
      Y2=YG2-YG3
      X1=XG1-XG3
      Y1=YG1-YG3
      A1=(X1-2*X2)*0.5
      B1=-X2
      A2=(Y1-2*Y2)*0.5
      B2=-Y2
    4 CONTINUE    
C
      DO 10 I=1,2
      DO 10 J=1,6
   10 GW(I,J)=0.
      A=A1**2 + A2**2 
      B=2*(A1*B1 + A2*B2) 
      C=B1**2 + B2**2 
      CONT1=(3-4*XNU)/(8.*3.1415926*GE*(1-XNU)) 
      CONT2=CONT1/(3-4*XNU)
      DO 250 I=1,10
      T1=((A1*GI(I)+B1)**2)/((A2*GI(I)+B2)**2 + (A1*GI(I)+B1)**2)
      T2=((A2*GI(I)+B2)**2)/((A2*GI(I)+B2)**2 + (A1*GI(I)+B1)**2)
      T3=(A1*GI(I)+B1)*(A2*GI(I)+B2)/((A1*GI(I)+B1)**2+(A2*GI(I)+B2)**2)
C
C  COMPUTE SHAPE FUNTIONS FOR NUMERICAL INTEGRATION
C
      F3=0.5*GI(I)*(GI(I)+1.)
      F2=1.-GI(I)**2
      F1=0.5*GI(I)*(GI(I)-1.)
      FL3=GIL(I)*(2.*GIL(I)-1.)
      FL2=4.*GIL(I)*(1.-GIL(I))
      FL1=(GIL(I)-1.)*(2.*GIL(I)-1.)
      FLN3=0.5*GIL(I)*(GIL(I)+1.)
      FLN2=1.-GIL(I)**2
      FLN1=0.5*GIL(I)*(GIL(I)-1.)
C
C  COMPUTE GW COEFFICIENTS
C
      GO TO(50,60,70) NODO
   50 XJA1=SQRT((4*A1*GIL(I)-2*A1+0.5*X3)**2+(4*A2*GIL(I)-2*A2+0.5*Y3)**
     *2)*2 
      XJA2=SQRT((A1*GI(I)*2+0.5*X3)**2+(A2*GI(I)*2+0.5*Y3)**2)
      XLO=-ALOG(2*SQRT((GI(I)*A1+B1)**2+(GI(I)*A2+B2)**2))
      S3=CONT1*(FL3*XJA1*OMEL(I)+F3*XJA2*XLO*OME(I)) 
      S2=CONT1*(FL2*XJA1*OMEL(I)+F2*XJA2*XLO*OME(I)) 
      S1=CONT1*(FL1*XJA1*OMEL(I)+F1*XJA2*XLO*OME(I)) 
      GO TO 200 
   60 XJA1=SQRT((0.5*B1-A1*GIL(I))**2+(0.5*B2-A2*GIL(I))**2)
      XJA11=SQRT((0.5*B1+A1*GIL(I))**2+(0.5*B2+A2*GIL(I))**2)
      XJA2=SQRT((0.5*B1+A1*GI(I))**2+(0.5*B2+A2*GI(I))**2)
      XLO=-0.5*ALOG((GI(I)*A1*0.5+B1*0.5)**2+(GI(I)*A2*0.5+B2*0.5)**2) 
      S3=CONT1*((FLN1*XJA1+FLN3*XJA11)*OMEL(I)+F3*XJA2*XLO*OME(I)) 
      S2=CONT1*(FLN2*(XJA1+XJA11)*OMEL(I)+F2*XJA2*XLO*OME(I))
      S1=CONT1*((FLN3*XJA1+FLN1*XJA11)*OMEL(I)+F1*XJA2*XLO*OME(I)) 
      GO TO 200 
   70 XJA1=SQRT((2*A1-4*A1*GIL(I)-0.5*X1)**2+(2*A2-4*A2*GIL(I)-0.5*Y1)**
     *2)*2
      XJA2=SQRT((2*A1*GI(I)-0.5*X1)**2+(2*A2*GI(I)-0.5*Y1)**2)
      XLO=-ALOG(2*SQRT((A1*GI(I)+B1)**2+(A2*GI(I)+B2)**2)) 
      S3=CONT1*(FL1*XJA1*OMEL(I)+F3*XJA2*XLO*OME(I))     
      S2=CONT1*(FL2*XJA1*OMEL(I)+F2*XJA2*XLO*OME(I))     
      S1=CONT1*(FL3*XJA1*OMEL(I)+F1*XJA2*XLO*OME(I))     
  200 GW(1,5)= GW(1,5)+S3+CONT2*F3*T1*XJA2*OME(I) 
      GW(1,3)= GW(1,3)+S2+CONT2*F2*T1*XJA2*OME(I) 
      GW(1,1)= GW(1,1)+S1+CONT2*F1*T1*XJA2*OME(I) 
      GW(1,6)= GW(1,6)+ CONT2*T3*XJA2*F3*OME(I) 
      GW(1,4)= GW(1,4)+ CONT2*T3*XJA2*F2*OME(I) 
      GW(1,2)= GW(1,2)+ CONT2*T3*XJA2*F1*OME(I) 
      GW(2,5)= GW(1,6)
      GW(2,3)= GW(1,4)
      GW(2,1)=GW(1,2) 
      GW(2,6)= GW(2,6)+ S3+CONT2*T2*XJA2*F3*OME(I)
      GW(2,4)=GW(2,4)+S2+CONT2*T2*XJA2*F2*OME(I)
      GW(2,2)= GW(2,2)+ S1+CONT2*T2*XJA2*F1*OME(I)
C
  250 CONTINUE
C
      RETURN
      END 
C------------------------------------------------------------------------
      SUBROUTINE SLNPD(A,B,D,N,NX)
C
C  PROGRAM 6 
C 
C SOLUTION OF LINEAR SYSTEMS OF EQUATIONS 
C BY THE GAUSS ELIMINATION METHOD PROVIDING 
C FOR INTERCHANGING ROWS WHEN ENCOUNTERING A
C ZERO DIAGONAL COEFICIENT
C 
C A : SYSTEM MATRIX 
C B : ORIGINALLY IT CONTAINS THE INDEPENDENT
C     COEFFICIENTS. AFTER SOLUTION IT CONTAINS
C     THE VALUES OF THE SYSTEM UNKNOWNS.
C 
C N : ACTUAL NUMBER OF UNKNOWNS 
C NX: ROW AND COLUMN DIMENSION OF A 
C 
      DIMENSION B(NX),A(NX,NX)
C
      TOL=1.E-6
C
      N1=N-1
      DO 100 K=1,N1 
      K1=K+1
      C=A(K,K)
      IF(ABS(C)-TOL)1,1,3
    1 DO 7 J=K1,N 
C 
C TRY TO INTERCHANGE ROWS TO GET NON ZERO DIAGONAL COEFFICIENT
C 
      IF(ABS((A(J,K)))-TOL)7,7,5 
    5 DO 6 L=K,N
      C=A(K,L)
      A(K,L)=A(J,L) 
    6 A(J,L)=C
      C=B(K)
      B(K)=B(J) 
      B(J)=C
      C=A(K,K)
      GO TO 3 
    7 CONTINUE
      GO TO 8
C 
C DIVIDE ROW BY DIAGONAL COEFFICIENT
C 
    3 C=A(K,K)
      DO 4 J=K1,N 
    4 A(K,J)=A(K,J)/C 
      B(K)=B(K)/C 
C 
C ELIMINATE UNKNOWN X(K) FROM ROW I 
C 
      DO 10 I=K1,N
      C=A(I,K)
      DO 9 J=K1,N 
    9 A(I,J)=A(I,J)-C*A(K,J)
   10 B(I)=B(I)-C*B(K)
  100 CONTINUE
C 
C COMPUTE LAST UNKNOWN
C 
      IF(ABS((A(N,N)))-TOL)8,8,101 
  101 B(N)=B(N)/A(N,N)
C 
C APPLY BACKSUBSTITUTION PROCESS TO COMPUTE REMAINING UNKNOWNS
C 
      DO 200 L=1,N1 
      K=N-L 
      K1=K+1
      DO 200 J=K1,N 
  200 B(K)=B(K)-A(K,J)*B(J) 
C 
C COMPUTE VALUE OF DETERMINANT
C 
      D=1.
      DO 250 I=1,N
  250 D=D*A(I,I)
      GO TO 300
    8 WRITE(IPR,2) K
    2 FORMAT(' **** SINGULARITY IN ROW',I5) 
      D=0.
  300 RETURN
      END 
C-----------------------------------------------------------------------  
      SUBROUTINE INTEREQ(FI,DFI,KODE,CX,CY,X,Y,SSOL,DSOL)
C
C  PROGRAM 41
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF STRESSES AND DISPLACEMENTS AT
C  INTERNAL POINTS.
C
      COMMON N,L,GE,XNU,INP,IPR
      DIMENSION FI(100),DFI(150),KODE(150),CX(20),CY(20)
      DIMENSION X(51),Y(51),SSOL(60),DSOL(40)
      DIMENSION HW(2,6),GW(2,6)
      DIMENSION D11(6),D12(6),D22(6),S11(6),S12(6),S22(6)
C
C  REARRANGE FI AND DFI ARRAYS TO STORE ALL THE VALUES OF THE
C  DISPLACEMENT IN FI AND ALL THE VALUES TRACTION IN DFI
C
      NE=N/2
      DO 180 I=1,NE
      DO 170 J=1,6
      IF(KODE(6*I-6+J)) 110,110,170
  110 IF((I-NE).NE.0 .OR. J.LT.5) GO TO 125
      IF(KODE(J-4)) 114,114,113
  113 CH=FI(J-4)*GE
      FI(J-4)=DFI(6*I-6+J)
      DFI(6*I-6+J)=CH
      GO TO 170
  114 DFI(6*I-6+J)=DFI(J-4)
      GO TO 170
  125 IF(I.EQ.1 .OR. J.GT.2 .OR. KODE(6*I-8+J).EQ.1) GO TO 130
      DFI(6*I-6+J)=DFI(6*I-8+J)
      GO TO 170
  130 CH=FI(4*I-4+J)*GE
      FI(4*I-4+J)=DFI(6*I-6+J)
      DFI(6*I-6+J)=CH
  170 CONTINUE
  180 CONTINUE
C
C  COMPUTE THE VALUES OF STRESSES AND DISPLACEMENTS AT INTERNAL POINTS   
C
      IF(L.EQ.0) GO TO 50
      DO 240 K=1,L
      SSOL(3*K-2)=0.
      SSOL(3*K-1)=0.
      SSOL(3*K)=0.
      DSOL(2*K-1)=0.
      DSOL(2*K)=0.
      DO 230 I=1,NE
      CALL EXTINEQ(CX(K),CY(K),X(2*I-1),Y(2*I-1),X(2*I),Y(2*I),
     1X(2*I+1),Y(2*I+1),HW,GW)
      CALL SIGMAEQ(CX(K),CY(K),X(2*I-1),Y(2*I-1),X(2*I),Y(2*I),
     1X(2*I+1),Y(2*I+1),D11,D12,D22,S11,S12,S22)
      DO 220 J=1,6
      IJ4=4*I-4+J
      IF(IJ4.GT.(4*NE)) IJ4=J-4
      SSOL(3*K-2)=SSOL(3*K-2)+D11(J)*DFI(6*I-6+J)-S11(J)*FI(IJ4)
      SSOL(3*K-1)=SSOL(3*K-1)+D12(J)*DFI(6*I-6+J)-S12(J)*FI(IJ4)
      SSOL(3*K)=SSOL(3*K)+D22(J)*DFI(6*I-6+J)-S22(J)*FI(IJ4)
      DSOL(2*K-1)=DSOL(2*K-1)+GW(1,J)*DFI(6*I-6+J)-HW(1,J)*FI(IJ4)
  220 DSOL(2*K)=DSOL(2*K)+GW(2,J)*DFI(6*I-6+J)-HW(2,J)*FI(IJ4)
  230 CONTINUE
  240 CONTINUE
   50 RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE SIGMAEQ(XP,YP,X1,Y1,X2,Y2,X3,Y3,D11,D12,D22,S11,S12
     *,S22) 
C
C  PROGRAM 42
C 
C  THIS SUBROUTINE COMPUTES THE VALUES OF THE S AND D MATRICES
C  USING GAUSS QUADRATURE IN ORDER TO COMPUTE THE STRESS
C  AT ANY INTERNAL POINT.  
C  RA           = RADIUS 
C  RD1,RD2,RDN  = RADIUS DERIVATIVES
C  ETA1,ETA2    = COMPONENTS OF THE UNIT NORMAL TO THE ELEMENT
C  XCO,YCO      = INTEGRATION POINT ALONG THE ELEMENT
C  XJA          = JACOBIAN
C
      COMMON N,L,GE,XNU,INP,IPR 
      DIMENSION D11(6),D12(6),D22(6),S11(6),S12(6),S22(6)
      DIMENSION GI(10),OME(10)
      DATA GI/0.9739065285,-0.9739065285,0.8650633666,-0.8650633666 
     @,0.6794095683,-0.6794095682,0.4333953941,-0.4333953941,
     @0.1488743389,-0.1488743389/                                    
      DATA OME/0.0666713443,0.0666713443,0.1494513491,0.1494513491
     @,0.2190863625,0.2190863625,0.2692667193,0.2692667193,
     @0.2955242247,0.2955242247/
      DO 20 J=1,6
      D11(J)=0.
      D12(J)=0.
      D22(J)=0.
      S11(J)=0.
      S12(J)=0.
   20 S22(J)=0.
C
      FA=1-4*XNU
      AL=1-2*XNU
      A=X3-2*X2+X1
      B=(X3-X1)/2
      C=Y3-2*Y2+Y1
      D=(Y3-Y1)/2 
      DE=4*3.141592*(1-XNU) 
      DO 40 I=1,10
C
C  COMPUTE VALUES OF THE SHAPE FUNCTIONS AT THE INTEGRATION POINTS
C
      F1=GI(I)*(GI(I)-1)*0.5
      F2=1.-GI(I)**2
      F3=GI(I)*(GI(I)+1)*0.5 
C
C  COMPUTE GEOMETRICAL PARAMETERS
C
      XCO=X1*F1+X2*F2+X3*F3
      YCO=Y1*F1+Y2*F2+Y3*F3
      XJA=SQRT((GI(I)*A+B)**2+(GI(I)*C+D)**2)
      ETA1=(GI(I)*C+D)/XJA
      ETA2=-(GI(I)*A+B)/XJA
      RA=SQRT((XP-XCO)**2+(YP-YCO)**2)
      RD1=(XCO-XP)/RA
      RD2=(YCO-YP)/RA
      RDN=RD1*ETA1+RD2*ETA2
C
C  COMPUTE D AND S COEFFICIENTS
C
      D11(1)=D11(1)+(AL*RD1+2*RD1**3)*OME(I)*XJA*F1/(DE*RA)  
      D11(2)=D11(2)+(2*RD1**2*RD2-AL*RD2)*OME(I)*XJA*F1/(DE*RA)
      D11(3)=D11(3)+(AL*RD1+2*RD1**3)*OME(I)*XJA*F2/(DE*RA)
      D11(4)=D11(4)+(2*RD1**2*RD2-AL*RD2)*OME(I)*XJA*F2/(DE*RA)
      D11(5)=D11(5)+(AL*RD1+2*RD1**3)*OME(I)*XJA*F3/(DE*RA)
      D11(6)=D11(6)+(2*RD1**2*RD2-AL*RD2)*OME(I)*XJA*F3/(DE*RA)
      D12(1)=D12(1)+(AL*RD2+2*RD1**2*RD2)*F1/(DE*RA)*OME(I)*XJA
      D12(2)=D12(2)+(AL*RD1+2*RD1*RD2**2)*F1/(DE*RA)*OME(I)*XJA
      D12(3)=D12(3)+(AL*RD2+2*RD1**2*RD2)*F2/(DE*RA)*OME(I)*XJA
      D12(4)=D12(4)+(AL*RD1+2*RD1*RD2**2)*F2/(DE*RA)*OME(I)*XJA
      D12(5)=D12(5)+(AL*RD2+2*RD1**2*RD2)*F3/(DE*RA)*OME(I)*XJA
      D12(6)=D12(6)+(AL*RD1+2*RD1*RD2**2)*F3/(DE*RA)*OME(I)*XJA
      D22(1)=D22(1)+(2*RD1*RD2**2-AL*RD1)*F1/(DE*RA)*OME(I)*XJA
      D22(2)=D22(2)+(AL*RD2+2*RD2**3)*F1/(DE*RA)*OME(I)*XJA
      D22(3)=D22(3)+(2*RD1*RD2**2-AL*RD1)*F2/(DE*RA)*OME(I)*XJA
      D22(4)=D22(4)+(AL*RD2+2*RD2**3)*F2/(DE*RA)*OME(I)*XJA
      D22(5)=D22(5)+(2*RD1*RD2**2-AL*RD1)*F3/(DE*RA)*OME(I)*XJA
      D22(6)=D22(6)+(AL*RD2+2*RD2**3)*F3/(DE*RA)*OME(I)*XJA
      S11(1)=S11(1)+(2*RDN*(AL*RD1+XNU*2*RD1-4*RD1**3)+4*XNU*ETA1
     1*RD1**2+AL*(2*ETA1*RD1**2+2*ETA1)-FA*ETA1)*2*GE*F1/(DE*RA**2)*
     2OME(I)*XJA
      S11(2)=S11(2)+(2*RDN*(AL*RD2-4*RD1**2*RD2)+4*XNU*ETA1*RD1*RD2+
     1AL*2*ETA2*RD1**2-FA*ETA2)*2*GE*F1/(DE*RA**2)*OME(I)*XJA
      S11(3)=S11(3)+(2*RDN*(AL*RD1+XNU*2*RD1-4*RD1**3)+4*XNU*ETA1
     1*RD1**2+AL*(2*ETA1*RD1**2+2*ETA1)-FA*ETA1)*2*GE*F2/(DE*RA**2)*
     2OME(I)*XJA
      S11(4)=S11(4)+(2*RDN*(AL*RD2-4*RD1**2*RD2)+4*XNU*ETA1*RD1*RD2+
     1AL*2*ETA2*RD1**2-FA*ETA2)*2*GE*F2/(DE*RA**2)*OME(I)*XJA
      S11(5)=S11(5)+(2*RDN*(AL*RD1+XNU*2*RD1-4*RD1**3)+4*XNU*ETA1
     1*RD1**2+AL*(2*ETA1*RD1**2+2*ETA1)-FA*ETA1)*2*GE*F3/(DE*RA**2)*
     2OME(I)*XJA
      S11(6)=S11(6)+(2*RDN*(AL*RD2-4*RD1**2*RD2)+4*XNU*ETA1*RD1*RD2+
     1AL*2*ETA2*RD1**2-FA*ETA2)*2*GE*F3/(DE*RA**2)*OME(I)*XJA
      S12(1)=S12(1)+(2*RDN*(XNU*RD2-4*RD1**2*RD2)+2*XNU*(ETA1*RD2*
     1RD1+ETA2*RD1**2)+AL*(2*ETA1*RD1*RD2+ETA2))*2*GE*F1/
     2(DE*RA**2)*OME(I)*XJA
      S12(2)=S12(2)+(2*RDN*(XNU*RD1-4*RD1*RD2**2)+2*XNU*(ETA1*RD2**2
     1+ETA2*RD1*RD2)+AL*(2*ETA2*RD1*RD2+ETA1))*2*GE*F1/
     2(DE*RA**2)*OME(I)*XJA
      S12(3)=S12(3)+(2*RDN*(XNU*RD2-4*RD1**2*RD2)+2*XNU*(ETA1*RD2*
     1RD1+ETA2*RD1**2)+AL*(2*ETA1*RD1*RD2+ETA2))*2*GE*F2/
     2(DE*RA**2)*OME(I)*XJA
      S12(4)=S12(4)+(2*RDN*(XNU*RD1-4*RD1*RD2**2)+2*XNU*(ETA1*RD2**2
     1+ETA2*RD1*RD2)+AL*(2*ETA2*RD1*RD2+ETA1))*2*GE*F2/
     2(DE*RA**2)*OME(I)*XJA
      S12(5)=S12(5)+(2*RDN*(XNU*RD2-4*RD1**2*RD2)+2*XNU*(ETA1*RD2*
     1RD1+ETA2*RD1**2)+AL*(2*ETA1*RD1*RD2+ETA2))*2*GE*F3/
     2(DE*RA**2)*OME(I)*XJA
      S12(6)=S12(6)+(2*RDN*(XNU*RD1-4*RD1*RD2**2)+2*XNU*(ETA1*RD2**2
     1+ETA2*RD1*RD2)+AL*(2*ETA2*RD1*RD2+ETA1))*2*GE*F3/
     2(DE*RA**2)*OME(I)*XJA
      S22(1)=S22(1)+(2*RDN*(AL*RD1-4*RD1*RD2**2)+4*XNU*ETA2*RD1*RD2+
     1AL*2*ETA1*RD2**2-FA*ETA1)*2*GE*F1/(DE*RA**2)*OME(I)*XJA
      S22(2)=S22(2)+(2*RDN*(AL*RD2+2*XNU*RD2-4*RD2**3)+4*XNU*ETA2
     1*RD2**2+AL*(2*ETA2*RD2**2+2*ETA2)-FA*ETA2)*2*GE*F1/
     2(DE*RA**2)*OME(I)*XJA
      S22(3)=S22(3)+(2*RDN*(AL*RD1-4*RD1*RD2**2)+4*XNU*ETA2*RD1*RD2+
     1AL*2*ETA1*RD2**2-FA*ETA1)*2*GE*F2/(DE*RA**2)*OME(I)*XJA
      S22(4)=S22(4)+(2*RDN*(AL*RD2+2*XNU*RD2-4*RD2**3)+4*XNU*ETA2
     1*RD2**2+AL*(2*ETA2*RD2**2+2*ETA2)-FA*ETA2)*2*GE*F2/
     2(DE*RA**2)*OME(I)*XJA
      S22(5)=S22(5)+(2*RDN*(AL*RD1-4*RD1*RD2**2)+4*XNU*ETA2*RD1*RD2+
     1AL*2*ETA1*RD2**2-FA*ETA1)*2*GE*F3/(DE*RA**2)*OME(I)*XJA
   40 S22(6)=S22(6)+(2*RDN*(AL*RD2+2*XNU*RD2-4*RD2**3)+4*XNU*ETA2
     1*RD2**2+AL*(2*ETA2*RD2**2+2*ETA2)-FA*ETA2)*2*GE*F3/
     2(DE*RA**2)*OME(I)*XJA
      RETURN
      END 
C-----------------------------------------------------------------------
      SUBROUTINE OUTPTEQ(X,Y,FI,DFI,CX,CY,SSOL,DSOL)
C
C  PROGRAM 43 
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE DISPLACEMENTS
C  AND TRACTIONS AT BOUNDARY NODES. IT ALSO PRINTS THE VALUES
C  OF DISPLACEMENTS AND STRESSES AT INTERNAL POINTS
C
      DIMENSION X(51),Y(51),FI(100),DFI(150)
      DIMENSION CX(20),CY(20),SSOL(60),DSOL(40) 
      COMMON N,L,GE,XNU,INP,IPR 
      NE=N/2
C
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//1X,'RESULTS'//2X,'BOUNDARY NODES'///10X
     1,'X',17X,'Y',11X,'DISPLACEMENT X',4X,'DISPLACEMENT Y'/)
      DO 10 I=1,N 
   10 WRITE(IPR,200) X(I),Y(I),FI(2*I-1),FI(2*I)
  200 FORMAT(4(4X,E14.7)) 
      WRITE(IPR,150)
  150 FORMAT(///28X,'TRACTION X',3X,'TRACTION Y',3X,'TRACTION X'
     1,3X,'TRACTION Y'/7X,'X',12X,'Y',7X,'BEFORE NODE',2X,'BEFORE NODE'
     2,2X,'AFTER NODE',3X,'AFTER NODE'/)                                     
      WRITE(IPR,250) X(1),Y(1),DFI(6*NE-1),DFI(6*NE),DFI(1),DFI(2)
      WRITE(IPR,250) X(2),Y(2),DFI(3),DFI(4),DFI(3),DFI(4)
      DO 15 I=2,NE
      WRITE(IPR,250) X(2*I-1),Y(2*I-1),DFI(6*I-7),DFI(6*I-6),
     1DFI(6*I-5),DFI(6*I-4)
   15 WRITE(IPR,250) X(2*I),Y(2*I),DFI(6*I-3),DFI(6*I-2),
     1DFI(6*I-3),DFI(6*I-2)
  250 FORMAT(6(1X,E12.5))
C
      IF(L.EQ.0) GO TO 30
      WRITE(IPR,300)
  300 FORMAT(///2X,'INTERNAL POINTS DISPLACEMENTS'//8X,'X',15X,'Y',9X  
     1,'DISPLACEMENT X',2X,'DISPLACEMENT Y')
      DO 20 K=1,L 
   20 WRITE(IPR,400)CX(K),CY(K),DSOL(2*K-1),DSOL(2*K)
      WRITE(IPR,350)
  350 FORMAT(//2X,'INTERNAL POINTS STRESSES'//8X,'X',15X,'Y',12X,
     1'SIGMA X',10X,'TAU XY',9X,'SIGMA Y')
      DO 25 K=1,L
   25 WRITE(IPR,450) CX(K),CY(K),SSOL(3*K-2),SSOL(3*K-1),SSOL(3*K)  
  400 FORMAT(4(2X,E14.7)) 
  450 FORMAT(5(2X,E14.7))
   30 WRITE(IPR,500)
  500 FORMAT(' ',79('*'))
      RETURN
      END 
